@@ -23,6 +23,7 @@
 #include <pyconfig.h>
 #include <alpm.h>
 #include <Python.h>
+#include "handle.h"
 #include "db.h"
 #include "package.h"
 #include "util.h"
@@ -30,6 +31,7 @@
 typedef struct _AlpmDB {
   PyObject_HEAD
   alpm_db_t *c_data;
+  PyObject *handle;
 } AlpmDB;
 
 #define ALPM_DB(self) (((AlpmDB*)self)->c_data)
@@ -37,17 +39,19 @@ typedef struct _AlpmDB {
 static PyTypeObject AlpmDBType;
 
 static void pyalpm_db_dealloc(AlpmDB *self) {
+  if (self->handle)
+    Py_DECREF(self->handle);
   Py_TYPE(self)->tp_free((PyObject*)self);
 }
 
-static PyObject* _pyobject_from_pmgrp(void *group) {
+static PyObject* _pyobject_from_pmgrp(void *group, PyObject *db) {
   const alpm_group_t* grp = (alpm_group_t*)group;
   if (!grp)
     Py_RETURN_NONE;
   else {
     PyObject *fst = PyUnicode_FromString(grp->name);
-    PyObject *snd = alpmlist_to_pylist(grp->packages,
-              pyalpm_package_from_pmpkg);
+    PyObject *snd = alpmlist_to_pylist2(grp->packages,
+              pyalpm_package_from_pmpkg, db);
     PyObject *tuple = PyTuple_Pack(2, fst, snd);
     Py_DECREF(fst);
     Py_DECREF(snd);
@@ -134,12 +138,12 @@ static int pyalpm_db_set_servers(PyObject* self, PyObject* value, void* closure)
 
 static PyObject* pyalpm_db_get_pkgcache(AlpmDB* self, void* closure) {
   alpm_list_t *pkglist = alpm_db_get_pkgcache(self->c_data);
-  return alpmlist_to_pylist(pkglist, pyalpm_package_from_pmpkg);
+  return alpmlist_to_pylist2(pkglist, pyalpm_package_from_pmpkg, self);
 }
 
 static PyObject* pyalpm_db_get_grpcache(AlpmDB* self, void* closure) {
   alpm_list_t *grplist = alpm_db_get_groupcache(self->c_data);
-  return alpmlist_to_pylist(grplist, _pyobject_from_pmgrp);
+  return alpmlist_to_pylist2(grplist, _pyobject_from_pmgrp, self);
 }
 
 /** Package get/set operations */
@@ -163,7 +167,7 @@ static PyObject* pyalpm_db_get_pkg(PyObject *rawself, PyObject* args) {
     Py_RETURN_NONE;
   } else {
     PyObject *result;
-    result = pyalpm_package_from_pmpkg(p);
+    result = pyalpm_package_from_pmpkg(p, self);
     if (result == NULL) {
       return NULL;
     } else {
@@ -182,18 +186,23 @@ static PyObject* pyalpm_db_get_group(PyObject* rawself, PyObject* args) {
   }
 
   grp = alpm_db_get_group(self->c_data, grpname);
-  return _pyobject_from_pmgrp(grp);
+  return _pyobject_from_pmgrp(grp, self);
 }
 
 static PyObject *pyalpm_db_update(PyObject *rawself, PyObject *args, PyObject *kwargs) {
   AlpmDB* self = (AlpmDB*)rawself;
+  alpm_db_t *db = ALPM_DB(self);
+  alpm_handle_t *handle = ALPM_HANDLE(self->handle);
+  alpm_list_t *dbs = NULL;
   char* keyword[] = {"force", NULL};
   int ret;
   PyObject *force;
   if (!PyArg_ParseTupleAndKeywords(args, kwargs, "O!", keyword, &PyBool_Type, &force))
     return NULL;
 
-  ret = alpm_db_update((force == Py_True), self->c_data);
+  dbs = alpm_list_add(dbs, db);
+  ret = alpm_db_update(handle, dbs, (force == Py_True));
+  alpm_list_free(dbs);
 
   switch(ret) {
   case -1:
@@ -210,12 +219,16 @@ static PyObject *pyalpm_db_update(PyObject *rawself, PyObject *args, PyObject *k
 static PyObject* pyalpm_db_search(PyObject *rawself, PyObject *args) {
   AlpmDB* self = (AlpmDB *)rawself;
   alpm_list_t* rawargs;
-  alpm_list_t* result;
+  alpm_list_t* result = NULL;
   int ok = pylist_string_to_alpmlist(args, &rawargs);
   if (ok == -1) return NULL;
 
-  result = alpm_db_search(self->c_data, rawargs);
-  return alpmlist_to_pylist(result, pyalpm_package_from_pmpkg);
+  ok = alpm_db_search(self->c_data, rawargs, &result);
+  FREELIST(rawargs);
+  // TODO: handle pm_errno being set and throw an exception
+  if (ok == -1) return NULL;
+
+  return alpmlist_to_pylist2(result, pyalpm_package_from_pmpkg, self);
 }
 
 static struct PyMethodDef db_methods[] = {
@@ -281,7 +294,7 @@ void init_pyalpm_db(PyObject *module) {
 }
 
 
-PyObject *pyalpm_db_from_pmdb(void* data) {
+PyObject *pyalpm_db_from_pmdb(void* data, PyObject *handle) {
   alpm_db_t *db = (alpm_db_t*)data;
   AlpmDB *self;
   self = (AlpmDB*)AlpmDBType.tp_alloc(&AlpmDBType, 0);
@@ -290,6 +303,11 @@ PyObject *pyalpm_db_from_pmdb(void* data) {
     return NULL;
   }
 
+  /* TODO: Remove check when src/package.c also has a refcount to a handle */
+  if (handle != NULL) {
+    Py_INCREF(handle);
+    self->handle = handle;
+  }
   self->c_data = db;
   return (PyObject *)self;
 }
@@ -343,7 +361,7 @@ PyObject* pyalpm_sync_get_new_version(PyObject *self, PyObject* args) {
   if (!result)
     Py_RETURN_NONE;
   else
-    return pyalpm_package_from_pmpkg(result);
+    return pyalpm_package_from_pmpkg(result, NULL);
 }
 
 /* vim: set ts=2 sw=2 et: */
